@@ -2,10 +2,6 @@
 # frozen_string_literal: true
 
 module TZInfo
-  # Use send as a workaround for erroneous 'wrong number of arguments' errors
-  # with JRuby 9.0.5.0 when calling methods with Java implementations. See #114.
-  send(:using, UntaintExt) if TZInfo.const_defined?(:UntaintExt)
-
   module DataSources
     # An {InvalidZoneinfoDirectory} exception is raised if {ZoneinfoDataSource}
     # is initialized with a specific zoneinfo path that is not a valid zoneinfo
@@ -78,6 +74,30 @@ module TZInfo
       DEFAULT_ALTERNATE_ISO3166_TAB_SEARCH_PATH = ['/usr/share/misc/iso3166.tab', '/usr/share/misc/iso3166'].freeze
       private_constant :DEFAULT_ALTERNATE_ISO3166_TAB_SEARCH_PATH
 
+      # Files and directories in the top level zoneinfo directory that will be
+      # excluded from the list of available time zones:
+      #
+      #   - +VERSION is included on Mac OS X.
+      #   - leapseconds is a list of leap seconds.
+      #   - localtime is the current local timezone (may be a link).
+      #   - posix, posixrules and right are directories containing other
+      #     versions of the zoneinfo files.
+      #   - SECURITY is included in the Arch Linux tzdata package.
+      #   - src is a directory containing the tzdata source included on Solaris.
+      #   - timeconfig is a symlink included on Slackware.
+      EXCLUDED_FILENAMES = [
+        '+VERSION',
+        'leapseconds',
+        'localtime',
+        'posix',
+        'posixrules',
+        'right',
+        'SECURITY',
+        'src',
+        'timeconfig'
+      ].freeze
+      private_constant :EXCLUDED_FILENAMES
+
       # Paths to be checked to find the system zoneinfo directory.
       #
       # @private
@@ -237,7 +257,10 @@ module TZInfo
         @timezone_identifiers = load_timezone_identifiers.freeze
         @countries = load_countries(iso3166_tab_path, zone_tab_path).freeze
         @country_codes = @countries.keys.sort!.freeze
-        @zoneinfo_reader = ZoneinfoReader.new(ConcurrentStringDeduper.new)
+
+        string_deduper = ConcurrentStringDeduper.new
+        posix_tz_parser = PosixTimeZoneParser.new(string_deduper)
+        @zoneinfo_reader = ZoneinfoReader.new(posix_tz_parser, string_deduper)
       end
 
       # Returns a frozen `Array` of all the available time zone identifiers. The
@@ -391,15 +414,7 @@ module TZInfo
       def load_timezone_identifiers
         index = []
 
-        # Ignoring particular files:
-        # +VERSION is included on Mac OS X.
-        # leapseconds is a list of leap seconds.
-        # localtime is the current local timezone (may be a link).
-        # posix, posixrules and right are directories containing other versions of the zoneinfo files.
-        # src is a directory containing the tzdata source included on Solaris.
-        # timeconfig is a symlink included on Slackware.
-
-        enum_timezones([], ['+VERSION', 'leapseconds', 'localtime', 'posix', 'posixrules', 'right', 'src', 'timeconfig']) do |identifier|
+        enum_timezones([], EXCLUDED_FILENAMES) do |identifier|
           index << identifier.join('/').freeze
         end
 
@@ -425,7 +440,7 @@ module TZInfo
           end
 
           unless entry =~ /\./ || exclude.include?(entry)
-            entry.untaint
+            RubyCoreSupport.untaint(entry)
             path = dir + [entry]
             full_path = File.join(@zoneinfo_dir, *path)
 
